# -*- tcl -*-
# $Id: lib_phone_common.tcl,v 1.2 2003/07/30 07:11:03 grigory Exp $

# if {$::tcl_platform(platform) == "windows"} {set DEVICE_PORT com4:}

proc error_ex {str} {
    puts "Error: $str"
    exit
}

namespace eval phonelib {
    variable PORT {}
    variable SPEED {}
    variable INIT_STR {}
    variable INIT_PHONE {}
    variable PHONE_MODEL {}

    variable f_phone {}
    variable f_phone_stat 0
    variable data {}
    variable rx_buf {}
    variable data_sended 0
    variable ring_str {}
    variable aft {}
}

proc phonelib::initialize {} {
    variable f_phone
    variable PHONE_MODEL

    set ping_res [send_data at]
    if {($f_phone == {}) || (($ping_res != 0) && ![regexp {OK} $ping_res])} {
	puts "Initializing..."
	# Whait, until port and phone be ready
	while {[init_phone] < 0} {}
	set num [operator_number]
	smslib::set_oper_number [lindex $num 0] [lindex $num 1]
	set PHONE_MODEL [phone_model]
	if {[if_gui]} {GUI::update}
    }
    after 30000 phonelib::initialize
}

proc phonelib::read_port {f} {
    variable rx_buf
    variable data_sended
    variable ring_str
    variable aft

    if {[eof $f]} { return }
    while {[string len [set buf [gets $f]]]} {
	append rx_buf $buf "\n"
#puts -nonewline .
	after cancel $aft
	set aft [after 5000 "set [namespace current]::data_sended -1"]
	if {[regexp -all {OK|ERROR|>} $buf]} {
	    set data_sended 1
	} elseif {[regexp -all {\+CMT: [0-9]+\n[0-9ABCDEF]+} $rx_buf]} {
	    regsub -all {^.*\+CMT: [0-9]+\n} $buf {} sms
	    regsub -all {\n} $sms {} sms
	    puts "Recieve SMS:\n$sms\n==========="
	    set sms [smslib::decode_sms $sms]
	    log $::RecivedFile $sms
	    if {[if_gui]} {after 0 "GUI::show_recived_msg [list $sms]"}
	    set rx_buf {}
	} elseif {[regexp -all $ring_str $buf]} {
	    puts "call/recieve: $rx_buf"
	    if {[set num [reg_number $rx_buf]] != -1} {
		set number [lindex $num 0]; set type [lindex $num 1]
		if {[string length $number] == 0} {set number Unknown}
		log $::IncomingCallLog "$number $type"
		if {[if_gui]} {after 0 "GUI::show_incoming_call $number $type"}
	    }
	    set rx_buf {}
	}
    }
}

proc phonelib::write_port {f} {
    variable data
    variable data_sended
    variable rx_buf
    if {[eof $f]} {set data_sended -1; return }
    if {[string len $data]} {
	set rx_buf {}
	if {$data != "at"} {puts "\nSend: '$data'"}
	# else {puts "\nPing device\n"}
	puts $f "$data"
	fileevent $f writable {}
    }
}

proc phonelib::open_port {port speed} {
    variable f_phone
    variable f_phone_stat
    variable PORT $port
    variable SPEED $speed
    if {$f_phone_stat > 0} { return }
    while {1} {
	puts "Opening port."
	if { ![catch {open $PORT r+} f_phone] } { break }
    }
    fconfigure $f_phone -mode $SPEED,n,8,1 -encoding utf-8 -buffering none -blocking off -translation crlf
    fileevent $f_phone readable [list [namespace current]::read_port $f_phone]
    set f_phone_stat 1
}

proc phonelib::init_phone {} {
    variable f_phone
    variable INIT_STR
    variable INIT_PHONE
    set ret 10
    puts "Init phone."
    if {$::tcl_platform(platform) == "windows"} {after 1000}
    close_port
    open_port $::PORT $::SPEED
    puts $f_phone $INIT_STR
    while {![regexp -all {OK} [gets $f_phone]] || ($ret < 0)} {incr ret -1; after 500}
    if {$ret <= 0} {error_ex "Phone init error."} else {
	foreach init_str $INIT_PHONE {puts [send_data $init_str]}
    }
    return $ret
}

proc phonelib::close_port {} {
    variable f_phone
    variable f_phone_stat
    if {$f_phone_stat > 0} {
	puts "Exit."
	catch {close $f_phone}
	set f_phone_stat 0
    }
}

namespace eval phonelib {
    variable send_data_flag 0
}
proc phonelib::send_data {send_command} {
    variable data
    variable f_phone
    variable rx_buf
    variable data_sended
    variable send_data_flag
    variable aft

    if {$send_data_flag != 0} { return 0 }
    set send_data_flag 1

    set data $send_command
    if {![catch {fileevent $f_phone writable [list [namespace current]::write_port $f_phone]}]} {
	set aft [after 5000 "set [namespace current]::data_sended -1"]
	vwait phonelib::data_sended
	after cancel $aft
	if {$data_sended < 0} {
	    puts "Sending error, due answere time out...";
	    set send_data_flag 0; return $data_sended
	}
	set res $rx_buf
	set rx_buf {}
	set data_sended 0
	set send_data_flag 0; return $res
    }
    set send_data_flag 0
    return -1
}

proc phonelib::melodies {} {
    set new_lst {}
    set melodies_str [send_data at*esom?]
    regsub -all {\*ESOM:|OK\n|\"} $melodies_str {} melodies_str
    set lst [split $melodies_str "\n"]
    for {set i 0} {$i<[llength $lst]} {incr i} {
	if {[string length [lindex $lst $i]]} {lappend new_lst [split [lindex $lst $i] ,]}
    }
    return $new_lst
}

proc phonelib::play_melody {num} {
    return [send_data at*erip=3,[expr $num + 30]]
}

proc phonelib::write_melody {numer melody} {
    regsub -all {\n} $melody {} melody
    return [send_data at*esom=$numer,\"$melody\",0]
}

proc phonelib::operator_number {} {
    regsub -all {\+CSCA:|OK|\n|\"| } [send_data at+csca?] {} number
    return [split $number ,]
}

proc phonelib::read_all_sms {} {
    variable cmd_sms_read
    set msgs {}
    set msg {}
    foreach sms [split [send_data $cmd_sms_read] "\n"] {
	if {[regexp {\+CMGL:} $sms]} {
	    if {$msg != {}} {lappend msgs $msg}
	    regsub {\+CMGL: |\n} $sms {} msg
	} elseif {[regexp {^[0-9]} $sms]} {
	    lappend msg [smslib::decode_sms $sms]
	}
    }
    if {$msg != {}} {lappend msgs $msg}
    return $msgs
}

proc phonelib::voice_answer {} {
    return [send_data $phonelib::cmd_answer]
}

proc phonelib::voice_busy {} {
    return [send_data $phonelib::cmd_busy]
}

proc phonelib::read_number {index} {
    variable cmd_phb_read
    set numbers [send_data $cmd_phb_read$index]
    set phones {}
    foreach number [split $numbers "\n"] {
	if {[regexp {^\+CPBR: } $number]} {
	    regsub -all {\+CPBR: |\"} $number {} number
	    lappend phones [split $number ,]
	}
    }
    return $phones
}

proc phonelib::read_all_numbers {} {
    variable cmd_phb_info

    puts "Getting phone book size."
    set book_info [send_data $cmd_phb_info]
    if {[string length $book_info] < 2} {puts {Error.}; return {}}
    regsub -all {\+CPBR: |\n.+|\(|\)} $book_info {} book_info
    set book_info [split $book_info ,]
    set book_size [split [lindex $book_info 0] -]
    puts "Book size: from=[lindex $book_size 0] to=[lindex $book_size 1]"

    puts "Reading phone book from phone."
    set phone_book [read_number [lindex $book_size 0],[lindex $book_size 1]]
    puts "\nRead '[llength $phone_book]' records from phone book."

    return $phone_book
}

proc phonelib::correct_number {number type} {
    if {$type == 145} { return +$number}
    return $number
}

# Initialize module
after 500 phonelib::initialize
