#!/usr/bin/tclsh
# -*- tcl -*-
# $Id: phonebooklib.tcl,v 1.6 2003/07/30 07:11:03 grigory Exp $

package provide phonebooklib 0.1

namespace eval phonebook {
    set PhoneBook {}
    set PhoneBookFile {}
}

proc phonebook::write_book_to_file {{book {}}} {
    variable PhoneBook
    if {$book != {}} {set PhoneBook $book}
    set f [open $::PhoneBookFile w+]
    foreach record $PhoneBook {puts $f $record}
    close $f
}

proc phonebook::read_book_from_file {} {
    variable PhoneBook {}
    if {[catch {set f [open $::PhoneBookFile r]}]} { return 1}
    while {![eof $f]} {lappend PhoneBook [gets $f]}
    close $f
    return 0
}

proc phonebook::names_list {} {
    variable PhoneBook
    set res {}
    foreach line $PhoneBook {
	lappend res "[lindex $line 3]"
    }
    return [lsort $res]
}

proc phonebook::get_number {name} {
    variable PhoneBook
    set res {}
    foreach line $PhoneBook {
	if {[regexp -all -nocase $name [lindex $line 3]]} {
	    set res $line
	    break
	}
    }
    if {$res == {}} {
	regsub {^\+|^0} $name {} phone
	if {[string is digit $phone]} { return $name}
	return {}
    }
    return [phonelib::correct_number [lindex $res 1] [lindex $res 2]]
}

proc phonebook::get_name {number} {
    variable PhoneBook
    regsub -all {\+} $number {} number
    set res [lindex [lsearch -inline -regexp $PhoneBook $number] 3]
    if {$res == {}} { return $number}
    return $res
}

#----------------------------------------
# Work with linked name-phone-type array
#----------------------------------------

proc phonebook::var_number {varname} {
    return [set varname](number)
}

proc phonebook::var_name {varname} {
    return [set varname](name)
}

proc phonebook::var_type {varname} {
    return [set varname](type)
}

proc phonebook::sync_by_name_add {varname {procname {}}} {
    if {![array exists [set varname]]} {
	array set [set varname] [list  procedure $procname number {} name {} type {}]
    }
    trace add variable $varname write [set [set varname](procedure)]
    return [set varname]
}

proc phonebook::sync_by_name_remove {varname {remove_var 1}} {
    if {[array exists $varname]} {
	if {[array names $varname procedure] != {}} {
	    trace remove variable $varname write [set [set varname](procedure)]
	}
	if {$remove_var} {
	    array unset $varname
	    if {[info exists $varname]} {unset $varname}
	}
    }
}

proc phonebook::sync_by_name {{name1 {}} {name2 {}} {op {}}} {
    sync_by_name_remove $name1 0
    switch $name2 {
	name   {set [set name1](number) [phonebook::get_number [set [set name1](name)]]}
	number {set [set name1](name) [phonebook::get_name [set [set name1](number)]]}
    }
    sync_by_name_add $name1
}
#----------------------------------------


proc phonebook::init_phonebook {} {
    read_book_from_file
    sync_by_name_add [namespace current]::SELECTED_RECORD [namespace current]::sync_by_name
}
